"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.runApp = void 0;
const koa_1 = __importDefault(require("koa"));
const koa_body_1 = __importDefault(require("koa-body"));
const config_1 = require("./config");
const db_1 = require("./db");
const router_1 = require("./router");
const utils_1 = require("./utils");
const chalk_1 = __importDefault(require("chalk"));
function runApp(configRootDir) {
    return __awaiter(this, void 0, void 0, function* () {
        yield config_1.loadConfig(configRootDir);
        const mongodbUriInfo = config_1.getConfig("mongodbConnectOption");
        if (mongodbUriInfo) {
            yield db_1.connectMongodb(mongodbUriInfo);
        }
        const koa = new koa_1.default();
        koa.proxy = true;
        const sendErrorResponse = (ctx, error) => {
            const result = {
                code: 500,
                message: "Internal server error",
            };
            utils_1.runWithDebugCheck(() => {
                console.error(error);
                result.data = error.message;
            });
            utils_1.sendHttpResponse(ctx, result);
        };
        koa.on("error", (error, ctx) => {
            sendErrorResponse(ctx, error);
        });
        const postBodyLimit = config_1.getConfig("postBodyLimit");
        koa.use(koa_body_1.default({
            jsonLimit: postBodyLimit,
            formLimit: postBodyLimit,
            textLimit: postBodyLimit,
            multipart: true,
        }));
        koa.use((ctx) => __awaiter(this, void 0, void 0, function* () {
            utils_1.runWithDebugCheck(() => {
                const method = ctx.method.toUpperCase();
                console.log(ctx.href, chalk_1.default.cyan(method));
                console.log("Headers", ctx.headers);
                if (method === "POST") {
                    console.log("Body", ctx.request.body);
                }
            });
            yield router_1.route(ctx);
        }));
        const port = config_1.getConfig("httpPort");
        const host = config_1.getConfig("httpHost");
        utils_1.runWithDebugCheck(() => {
            console.log(`Start Http Service -> ` + chalk_1.default.blue(`${host}:${port}`));
        });
        koa.listen(port, host);
    });
}
exports.runApp = runApp;
