"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createMongooseModel = exports.disconnectMongodb = exports.connectMongodb = exports.createConnectionURI = void 0;
const mongoose_1 = __importDefault(require("mongoose"));
const utils_1 = require("./utils");
const chalk_1 = __importDefault(require("chalk"));
function createConnectionURI(option) {
    var _a;
    let config = {
        host: "127.0.0.1",
        port: 27017,
        debug: false,
    };
    if (typeof option === "object") {
        config = Object.assign(Object.assign({}, config), option);
    }
    let uri = "mongodb://";
    if (config.user) {
        uri += `${config.user}:${(_a = config.password) !== null && _a !== void 0 ? _a : ""}@`;
    }
    uri += `${config.host}:${config.port}`;
    if (config.database) {
        uri += `/${config.database}`;
    }
    return uri;
}
exports.createConnectionURI = createConnectionURI;
function connectMongodb(option) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!!(option === null || option === void 0 ? void 0 : option.debug)) {
            mongoose_1.default.set("debug", true);
        }
        const uri = createConnectionURI(option);
        mongoose_1.default.connect(uri, {
            useNewUrlParser: true,
            keepAlive: true,
            useUnifiedTopology: true,
            useCreateIndex: true,
            useFindAndModify: false,
        });
        yield new Promise((resolve, reject) => {
            const db = mongoose_1.default.connection;
            db.on("error", () => {
                db.close();
                reject(`An error occurred while connecting to the database via '${uri}'`);
            });
            db.once("open", resolve);
        });
        utils_1.runWithDebugCheck(() => {
            console.log(`Connected to mongodb database -> ` + chalk_1.default.blue(uri));
        });
    });
}
exports.connectMongodb = connectMongodb;
function disconnectMongodb() {
    return __awaiter(this, void 0, void 0, function* () {
        yield mongoose_1.default.connection.close();
    });
}
exports.disconnectMongodb = disconnectMongodb;
function createMongooseModel(name, definition, option) {
    let schemaOption = {
        timestamps: true,
    };
    if (typeof option === "object") {
        schemaOption = Object.assign(Object.assign({}, schemaOption), option);
    }
    const schema = new mongoose_1.default.Schema(definition, schemaOption);
    const model = mongoose_1.default.model(name, schema, name);
    return { schema, model };
}
exports.createMongooseModel = createMongooseModel;
