"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.route = exports.RouteError = void 0;
const config_1 = require("./config");
const path_1 = __importDefault(require("path"));
const fs_1 = __importDefault(require("fs"));
const utils_1 = require("./utils");
const sprintf_js_1 = require("sprintf-js");
const chalk_1 = __importDefault(require("chalk"));
exports.RouteError = {
    InterfacePathIllegal: `Path %s must contain controller and action information`,
    ControllerFileNotExist: `Controller file %s does not exist`,
    ControllerNotExist: `The file %s does not exist for the default exported controller`,
    ActionNotExist: `Action %s does not exist`,
    ServerHasNoOutput: "No response content from the server",
};
function sendErrorResponse(ctx, message) {
    const result = {
        code: 404,
        message: "404 Not Found",
    };
    utils_1.runWithDebugCheck(() => {
        console.error("Error:", chalk_1.default.red(message));
        result.data = message;
    });
    utils_1.sendHttpResponse(ctx, result);
}
const controllerNameMap = {};
const actionNameMap = {};
function route(ctx) {
    return __awaiter(this, void 0, void 0, function* () {
        const routePath = ctx.path.replace(/^\/*|\/*$/g, "");
        const segments = routePath.split("/");
        if (segments.length < 2) {
            return sendErrorResponse(ctx, sprintf_js_1.sprintf(exports.RouteError.InterfacePathIllegal, ctx.path));
        }
        const rootDir = config_1.getConfig("controllerRootDir");
        let actionName = segments.pop();
        let controllerName = segments.pop();
        const controllerDir = path_1.default.resolve(rootDir, segments.join(path_1.default.sep));
        if (!controllerNameMap[controllerDir]) {
            controllerNameMap[controllerDir] = new Map();
            const fileList = fs_1.default.readdirSync(controllerDir);
            for (let file of fileList) {
                controllerNameMap[controllerDir].set(file.toLowerCase(), file);
            }
        }
        const controllerFile = path_1.default.resolve(rootDir, `./${controllerNameMap[controllerDir].get(controllerName.toLowerCase() + ".js")}`);
        if (!fs_1.default.existsSync(controllerFile)) {
            return sendErrorResponse(ctx, sprintf_js_1.sprintf(exports.RouteError.ControllerFileNotExist, controllerFile));
        }
        let controllerClass = (yield Promise.resolve().then(() => __importStar(require(controllerFile)))).default;
        if (typeof controllerClass !== "function") {
            return sendErrorResponse(ctx, sprintf_js_1.sprintf(exports.RouteError.ControllerNotExist, controllerFile));
        }
        const controller = new controllerClass(ctx);
        if (!actionNameMap[controllerFile]) {
            actionNameMap[controllerFile] = new Map();
            const propertyNames = [
                ...Object.getOwnPropertyNames(controller),
                ...Object.getOwnPropertyNames(Object.getPrototypeOf(controller)),
            ];
            for (let key of propertyNames) {
                if (key === "constructor") {
                    continue;
                }
                if (typeof controller[key] === "function") {
                    actionNameMap[controllerFile].set(key.toLowerCase(), key);
                }
            }
        }
        const rawActionName = actionName;
        actionName = actionNameMap[controllerFile].get(actionName.toLowerCase());
        if (!actionName) {
            return sendErrorResponse(ctx, sprintf_js_1.sprintf(exports.RouteError.ActionNotExist, rawActionName));
        }
        const onBeforeActionHook = config_1.getConfig("onBeforeActionHook");
        if (typeof controller[onBeforeActionHook] === "function") {
            const beforeHookResult = yield controller[onBeforeActionHook]();
            if (beforeHookResult === "prevent") {
                return;
            }
        }
        yield controller[actionName]();
        const onAfterActionHook = config_1.getConfig("onAfterActionHook");
        if (typeof controller[onAfterActionHook] === "function") {
            yield controller[onAfterActionHook]();
        }
        if (!ctx.body) {
            utils_1.sendHttpResponse(ctx, { message: exports.RouteError.ServerHasNoOutput });
        }
    });
}
exports.route = route;
